//
// PlainDirection.java
//

public class PlainDirection {
    // OVERVIEW: Immutable type for representing a compass direction.
    //   A direction is one of: N, E, S, W.

    // Rep:
    private int northsouth; 
    private int eastwest;

    // RI (c) =
    //      (c.northsouth == -1 || c.northsouth == 0 || c.northsouth == 1)
    //   && (c.eastwest == -1 || c.eastwest == 0 || c.eastwest == 1)
    //   && (c.northsouth != 0 || c.eastwest != 0)

    //@invariant northsouth >= -1 && northsouth <= 1
    //@invariant eastwest >= -1 && eastwest <= 1
    //@invariant northsouth != 0 || eastwest != 0
    //@invariant northsouth == 0 || eastwest == 0

    static final public PlainDirection NORTH = new PlainDirection (1, 0);
    static final public PlainDirection EAST = new PlainDirection (0, 1);
    static final public PlainDirection SOUTH = new PlainDirection (-1, 0);
    static final public PlainDirection WEST = new PlainDirection (0, -1);

    private PlainDirection (int ns, int ew)
	//@requires ns >= -1 && ns <= 1
	//@requires ew >= -1 && ew <= 1
	//@requires ns != 0 || ew != 0
	//@requires ns == 0 || ew == 0
	// EFFECTS: Creates a new direction with northerly direction ns and easterly direction ew.
	//   Note: this is private.  Clients can only use the direction constants.
    {
	northsouth = ns;
	eastwest = ew;
    }

    public PlainDirection leftDirection () {
	if (this == NORTH) {
	    return WEST;
	} else if (this == WEST) {
	    return SOUTH;
	} else if (this == SOUTH) {
	    return EAST;
	} else if (this == EAST) {
	    return NORTH;
	} else {
	    throw new RuntimeException ("ERROR: Bad direction: " + this);
	}
    } //@nowarn Exception

    public PlainDirection rightDirection () {
	if (this == NORTH) {
	    return EAST;
	} else if (this == WEST) {
	    return NORTH;
	} else if (this == SOUTH) {
	    return WEST;
	} else if (this == EAST) {
	    return SOUTH;
	} else {
	    throw new RuntimeException ("ERROR: Bad direction: " + this);
	}
    } //@nowarn Exception

    public String toString ()
    {
	String res;

	switch (northsouth) 
	    {
	    case 1:  res = "north"; break;
	    case -1: res = "south"; break;
	    default: res = ""; break;
	    }

	switch (eastwest) 
	    {
	    case 1: res += "east"; break;
	    case -1: res += "west"; break;
	    default: break;
	    }

	return res;
    }

    public int northerlyDirection () {
	return northsouth;
    }

    public int easterlyDirection () {
	return eastwest;
    }

    static public PlainDirection fromChar (char d) {
	switch (d) {
	case 'n': case 'N': return NORTH;
	case 's': case 'S': return SOUTH;
	case 'e': case 'E': return EAST;
	case 'w': case 'W': return WEST;
	}

	throw new RuntimeException ("Bad direction: " + d);
    } //@nowarn Exception


    public char toChar () {
	if (this == NORTH) {
	    return 'n';
	} else if (this == WEST) {
	    return 'w';
	} else if (this == SOUTH) {
	    return 's';
	} else if (this == EAST) {
	    return 'e';
	} else {
	    throw new RuntimeException ("Bad direction: " + this);
	}
    } //@nowarn Exception
}
