//
// SnakeInterface.java
//
// You may not modify this code.
//
// See SnakeInterface.spec for the specification.
//

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.net.*;
import java.io.*;

abstract public class SnakeInterface extends Thread {
    private int score = 0;
    private int id = -1;
    PlainDirection direction = PlainDirection.NORTH;
    private Socket socket;
    /*@non_null@*/ private BufferedReader in;
    /*@non_null@*/ private PrintStream out;
    private Coordinate size;
    private boolean running = true;
    private Vector snake;
    
    private Vector apples;
    private Vector othersnakes;
    
    //@invariant snake != null
    //@invariant snake.elementType == \type (Coordinate)
    //@invariant snake.containsNull == false

    //@invariant apples != null
    //@invariant apples.elementType == \type (Coordinate)
    //@invariant apples.containsNull == false;

    //@invariant othersnakes != null
    //@invariant othersnakes.elementType == \type (Vector)
    //@invariant othersnakes.containsNull == true; 
    // the snake entry matching myself will be null
    
    public SnakeInterface (/*@non_null@*/ Socket sock) throws IOException {
	snake = new Vector ();
	//@set snake.elementType = \type(Coordinate)
	//@set snake.containsNull = false

	apples = new Vector ();
	//@set apples.elementType = \type(Coordinate)
	//@set apples.containsNull = false

	othersnakes = new Vector ();
	//@set othersnakes.elementType = \type(Vector) //? this doesn't seem to work
	//@set othersnakes.containsNull = true;

	size = null;
	socket = sock;
	
	in = new BufferedReader (new InputStreamReader (socket.getInputStream ()));
	out = new PrintStream (socket.getOutputStream ());
	this.start ();

	// Don't return until the board size is set!
	// This is quite dangerous...snake will hang if server is broken.
	while (size == null) {
	    System.err.println ("Waiting for board size...");
	    try {
		Thread.sleep (100);
	    } catch (InterruptedException ie) {
		;
	    }
	}
    } //@nowarn Invariant

    //@requires vec.elementType == \type(Coordinate)
    private void readCoordinates (/*@non_null@*/ String input, /*@non_null@*/ Vector vec) throws RuntimeException
    {
	// System.err.println ("Reading coordinates: " + input);
	StringTokenizer st = new StringTokenizer (input, ";");
	while (st.hasMoreTokens ()) {
	    String next = st.nextToken ();
	    StringTokenizer st2 = new StringTokenizer (next, ",");

	    if  (st2.countTokens () > 1) {
		try {
		    int x = Integer.valueOf (st2.nextToken ()).intValue (); 
		    int y = Integer.valueOf (st2.nextToken ()).intValue (); 
		    vec.add (new Coordinate (x, y));
		} catch (NumberFormatException e) {
		    System.err.println ("Number format exception: " + next + " / " + e);
		    throw new RuntimeException ("ERROR: bad coordinate input: " + input);
		}
	    } else {
		throw new RuntimeException ("ERROR: bad coordinate input: " + input);
	    }
	}
    }

    public void run () throws RuntimeException { 
	String input;

	try {
	    input = in.readLine ();
	} catch (IOException e) {
	    throw new RuntimeException ("Error reading data from server: " + e);
	}

	//
	// Server messages:
	//
	// message ::=
        //     s<int>           % this snake's score
        //   | d<dir>           % this snake's direction (nsew)
	//   | r                % this snake is running
	//   | x                % this snake is not running
	//   | i:<int>          % this snake's id 
	//   | b:<int>,<int>    % size of the board (rows, columns)
	//   | u:<coords>       % complete position of this snake (head to tail)
	//   | a:<coords>       % positions of all apples
	//   | t:<int>:<coords> % position of snake <int> (head to tail)
	//
	// coords ::=
        //     <int>,<int>;<coords>
	//   | % empty
	//
	// dir ::= n | s | e | w
	// 
	// int ::= [0-9]*
	//

	while (input != null) {
	    // System.err.println ("Received message: " + input);

	    if (input.length () == 1) {
		switch  (input.charAt (0)) {
		case 'r':
		    running = true;
		    break;
		case 'x':
		    running = false;
		    break;
		default:
		    throw new RuntimeException ("Unhandled server code: " + input);
		}
	    } else {
		if (input.length () < 2) {
		    throw new RuntimeException ("Bad server message received: " + input);
		} else {
		    char code = input.charAt (0);
		    char colon = input.charAt (1);

		    if (colon != ':') {
			throw new RuntimeException ("Bad server message received (no colon): " + input);
		    }

		    String rest = input.substring (2);
		    
		    switch  (code) {
		    case 's':
			score = Integer.valueOf (rest).intValue ();
			break;
		    case 'i':
			id = Integer.valueOf (rest).intValue ();
			break;
		    case 'd':
			direction = PlainDirection.fromChar (rest.charAt (0));
			break;
		    case 'b':
			{
			    StringTokenizer st = new StringTokenizer (rest, ",");
			    size = new Coordinate (Integer.valueOf (st.nextToken ()).intValue (),
						   Integer.valueOf (st.nextToken ()).intValue ());
			    System.err.println ("Got board size: " + size);
			    break;
			}
		    case 'u':
			synchronized  (snake) {
			    snake.clear ();
			    readCoordinates (rest, snake);
			}
			break;
		    case 'a':
			synchronized  (apples) {
			    apples.clear ();
			    readCoordinates (rest, apples);
			}
			break;
		    case 't':
			synchronized  (othersnakes) {
			    try {
				StringTokenizer st = new StringTokenizer (rest, ":");
				int snakeno = Integer.valueOf (st.nextToken ()).intValue ();

				try {
				    String coords = st.nextToken ();
				    // System.err.println ("Reading snake: " + snakeno + ": " + coords);
				    
				    while (snakeno >= othersnakes.size ()) {
					// System.err.println ("Adding element: " + othersnakes.size ());
					othersnakes.add (new Vector ());
				    }
				    
				    if (snakeno == getId ()) {
					System.err.println ("ERROR: Server should not send self snake.");
				    } else {
					// Need the nowarns: we know every snake except self is non-null
					((Vector) othersnakes.elementAt (snakeno)).setSize (0); //@nowarn Pre, Null
					readCoordinates (coords, (Vector) othersnakes.elementAt (snakeno)); //@nowarn Pre, NonNull
				    }
				} catch (NoSuchElementException e) {
				    // No coordinates, the snake is dead
				    ((Vector) othersnakes.elementAt (snakeno)).setSize (0); //@nowarn Pre, Null
				}
			    } catch (NumberFormatException nfe) {
				System.err.println ("Bad server message: " + input);
			    }
			}
			break;
		    default:
			throw new RuntimeException ("Unhandled server code: " + input);
		    }
		}
	    }

	    updateState ();

	    try {
		input = in.readLine ();
	    } catch (IOException e) {
		System.err.println ("Error reading data from server: " + e);
	    }
	}
    }

    // Abstract method that will be called whenever the state of the world has changed.
    abstract public void updateState ();

    public void startGame () {
	out.println ("s");
    }

    public void setSnakeName (String name) {
	out.println ("n:" + name);
    }
    
    public void turnLeft () {
	out.println ("l");
    }
    
    public void turnRight () {
	out.println ("r");
    }
    
    public int getScore () {
	return score;
    }
    
    public boolean isPlaying () {
	return running;
    }
    
    public PlainDirection getDirection () {
	return direction;
    }
    
    public Coordinate getBoardSize () throws RuntimeException {
	if (size == null) {
	    throw new RuntimeException ("Board size is not yet known!");
	} else {
	    return size;
	}
    }
    
    public Vector getSnake () {
	return snake;
    }
    
    //@ensures \result >= 0
    public int getId () throws RuntimeException {
	if (id < 0) {
	    throw new RuntimeException ("Id is not yet known!");
	}

	return id;
    } 

    public Vector getApples () {
	return apples;
    }
    
    public int getNumberOfSnakes () {
	return othersnakes.size () - 1; // zeroth element is not used
    }

    //@requires n >= 0
    public Vector getNthSnake (int n) {
	// REQUIRES: n < the number of snakes in the snake pit.
	// EFFECTS: Returns an ordered Vector that contains all the
	//    Coordinates in the snake pit that are currently occupied
	//    by snake number n.  The snakes in the snake pit are numbered
	//    sequentially starting from 1.  If the requested snake is dead, 
	//    returns an empty vector.

	return (Vector) othersnakes.elementAt (n); //@nowarn Pre // runtime error if requires is not satisfied
    }
}
